/* $Id: ast_ehandlers.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_EHANDLERS_H
#define WTPARSER_AST_EHANDLERS_H

#include "../astnode.h"
#include <deque>

namespace wtast
{

class EHandler;

class EHandlers : public AstNode
{
	typedef std::deque<EHandler*> ehandlers_t;
	ehandlers_t mEHandlers;

public:
	EHandlers()
		: AstNode()
	{
		// ok
	}

	template<typename Iter>
	EHandlers(const Iter & first, const Iter & last)
		: AstNode()
		, mEHandlers(first, last)
	{
		// ok
	}

	~EHandlers();

	void Accept(AstVisitor* vis);

	EHandlers* Copy();

	Kind GetKind() const
	{
		return EHANDLERS;
	}

	std::size_t GetNumberOfEHandlers() const
	{
		return mEHandlers.size();
	}

	EHandler* GetEHandler(std::size_t idx)
	{
		if(idx<mEHandlers.size())
		{
			return mEHandlers[idx];
		}
		else
		{
			return 0;
		}
	}
};

} // namespace wtast

#endif // WTPARSER_AST_EHANDLERS_H

