/* $Id: ast_seqliteral.cpp 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "ast_seqliteral.h"
#include "ast_seqtype.h"
#include "../astvisitor.h"

wtast::SeqLiteral::~SeqLiteral()
{
	delete mInitializer;
}

void wtast::SeqLiteral::Accept(AstVisitor* vis)
{
	if(vis->Visit(this))
	{
		if(mInitializer)
		{
			mInitializer->Accept(vis);
		}

		vis->EndVisit(this);
	}
}

bool wtast::SeqLiteral::OriginateFrom(Variable::Class varclass)
{
	if(mInitializer && mInitializer->OriginateFrom(varclass))
	{
		return true;
	}

	return Variable::READONLY==varclass || Variable::LOCAL==varclass;
}

wtast::SeqLiteral* wtast::SeqLiteral::Copy()
{
	Expression* init = 0;

	if(mInitializer)
	{
		init = mInitializer->Copy();
	}

	return new SeqLiteral(mType, init);
}

wtast::Type* wtast::SeqLiteral::GetElementType()
{
	return mType->GetElementType();
}

wtast::TypeInfo* wtast::SeqLiteral::GetType()
{
	return static_cast<TypeInfo*>(mType);
}
