/* $Id: ast_variableaccess.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_VARIABLEACCESS_H
#define WTPARSER_AST_VARIABLEACCESS_H

#include "../astnode.h"
#include "../uniqueid.h"
#include "ast_lvalue.h"
#include <string>

namespace wtast
{

class Type;

class VariableAccess : public LValue
{
	Type*              mType;
	std::string        mName;
	std::string        mFrameName;
	wtparser::UniqueId mUniqueId;
	Variable::Class    mVarClass;

public:
	VariableAccess(Type* type, const std::string & name, const std::string & frame_name, wtparser::UniqueId unique_id, Variable::Class varclass)
		: LValue()
		, mType(type)
		, mName(name)
		, mFrameName(frame_name)
		, mUniqueId(unique_id)
		, mVarClass(varclass)
	{
		// ok
	}

	~VariableAccess();

	void Accept(AstVisitor* vis);

	VariableAccess* Copy();

	Kind GetKind() const
	{
		return VARIABLEACCESS;
	}

	Variable::Class GetVarClass() const
	{
		return mVarClass;
	}

	bool OriginateFrom(Variable::Class varclass)
	{
		return mVarClass==varclass;
	}

	std::string & GetName()
	{
		return mName;
	}

	std::string & GetFrameName()
	{
		return mFrameName;
	}

	wtparser::UniqueId GetUniqueId()
	{
		return mUniqueId;
	}

	TypeInfo* GetType();
};

} // namespace wtast

#endif // WTPARSER_AST_VARIABLEACCESS_H
