//================================================================================================
// $Id: astvisitor.h 6309 2023-01-16 20:19:42Z aron $
//
// Project:
//		Welltype Parser
// Contents:
//		AstVisitor
// Authors:
//		Aron Barath, 2014-2023
//================================================================================================
// Copyright (c) Aron Barath
//================================================================================================

#ifndef WTPARSER_ASTVISITOR_H
#define WTPARSER_ASTVISITOR_H

namespace wtast
{

class Program;
class Metadata;
class MainBlock;
class ConstructorBlock;
class DestructorBlock;
class DeclareBlock;
class EHandlers;
class EHandler;
class Finally;
class Compound;
class ExceptionCompound;
class VariableDeclaration;
class VariableDeclFragment;
class VariableAccess;
class ElementAccess;
class BasicType;
class SeqType;
class SimultaneousType;
class RecordType;
class RecordField;
class RecordDeclaration;
class SimultaneousExpression;
class StringLiteral;
class BooleanLiteral;
class CharacterLiteral;
class NumberLiteral;
class Assignment;
class FunctionSignature;
class FunctionDefinition;
class FunctionArgument;
class FunctionDeclaration;
class FunctionCall;
class ProcedureCall;
class ExceptionDeclaration;
class PreCondition;
class PostCondition;
class MemberAccess;
class AndThen;
class OrElse;
class ReturnStatement;
class OperatorDeclaration;
class RaiseStatement;
class WhileStatement;
class DoWhileStatement;
class ForStatement;
class ForeachStatement;
class IfStatement;
class GlobalBlock;
class AssertStatement;
class SeqLiteral;
class RecordLiteral;
class FieldAssign;
class DataType;
class DataTypeDeclaration;
class DataTypeCons;
class DataTypeLiteral;
class SwitchStatement;
class DataTypeCaseStatement;
class DefaultStatement;
class FunctionType;
class FunctionTypeDeclaration;
class FunctionPointerCall;
class MakeFunctionPointer;
class UnresolvedFunctionName;
class Enumerator;
class EnumType;
class EnumTypeDeclaration;
class EnumCaseStatement;
class EnumLiteral;
class RecordUpdate;

class AstVisitor
{
	const bool _dive;
public:
	AstVisitor(bool dive=true) : _dive(dive) { }
	virtual ~AstVisitor() { }

	virtual bool Visit(Program*) { return _dive; }
	virtual void EndVisit(Program*) { }

	virtual bool Visit(MainBlock*) { return _dive; }
	virtual void EndVisit(MainBlock*) { }

	virtual bool Visit(ConstructorBlock*) { return _dive; }
	virtual void EndVisit(ConstructorBlock*) { }

	virtual bool Visit(DestructorBlock*) { return _dive; }
	virtual void EndVisit(DestructorBlock*) { }

	virtual bool Visit(DeclareBlock*) { return _dive; }
	virtual void EndVisit(DeclareBlock*) { }

	virtual bool Visit(Metadata*) { return false; }

	virtual bool Visit(EHandlers*) { return _dive; }
	virtual void EndVisit(EHandlers*) { }

	virtual bool Visit(EHandler*) { return _dive; }
	virtual void EndVisit(EHandler*) { }

	virtual bool Visit(Finally*) { return _dive; }
	virtual void EndVisit(Finally*) { }

	virtual bool Visit(Compound*) { return _dive; }
	virtual void EndVisit(Compound*) { }

	virtual bool Visit(ExceptionCompound*) { return _dive; }
	virtual void EndVisit(ExceptionCompound*) { }

	virtual bool Visit(VariableDeclaration*) { return _dive; }
	virtual void EndVisit(VariableDeclaration*) { }

	virtual bool Visit(VariableDeclFragment*) { return false; }

	virtual bool Visit(VariableAccess*) { return false; }

	virtual bool Visit(ElementAccess*) { return _dive; }
	virtual void EndVisit(ElementAccess*) { }

	virtual bool Visit(BasicType*) { return false; }

	virtual bool Visit(SeqType*) { return _dive; }
	virtual void EndVisit(SeqType*) { }

	virtual bool Visit(SimultaneousType*) { return _dive; }
	virtual void EndVisit(SimultaneousType*) { }

	virtual bool Visit(RecordType*) { return _dive; }
	virtual void EndVisit(RecordType*) { }

	virtual bool Visit(RecordDeclaration*) { return _dive; }
	virtual void EndVisit(RecordDeclaration*) { }

	virtual bool Visit(RecordField*) { return _dive; }
	virtual void EndVisit(RecordField*) { }

	virtual bool Visit(SimultaneousExpression*) { return _dive; }
	virtual void EndVisit(SimultaneousExpression*) { }

	virtual bool Visit(StringLiteral*) { return false; }

	virtual bool Visit(BooleanLiteral*) { return false; }

	virtual bool Visit(CharacterLiteral*) { return false; }

	virtual bool Visit(NumberLiteral*) { return false; }

	virtual bool Visit(Assignment*) { return _dive; }
	virtual void EndVisit(Assignment*) { }

	virtual bool Visit(FunctionSignature*) { return _dive; }
	virtual void EndVisit(FunctionSignature*) { }

	virtual bool Visit(FunctionArgument*) { return _dive; }
	virtual void EndVisit(FunctionArgument*) { }

	virtual bool Visit(FunctionDeclaration*) { return _dive; }
	virtual void EndVisit(FunctionDeclaration*) { }

	virtual bool Visit(FunctionDefinition*) { return _dive; }
	virtual void EndVisit(FunctionDefinition*) { }

	virtual bool Visit(FunctionCall*) { return _dive; }
	virtual void EndVisit(FunctionCall*) { }

	virtual bool Visit(ProcedureCall*) { return _dive; }
	virtual void EndVisit(ProcedureCall*) { }

	virtual bool Visit(ExceptionDeclaration*) { return false; }

	virtual bool Visit(OperatorDeclaration*) { return false; }

	virtual bool Visit(RaiseStatement*) { return false; }

	virtual bool Visit(UnresolvedFunctionName*) { return false; }

	virtual bool Visit(PreCondition*) { return _dive; }
	virtual void EndVisit(PreCondition*) { }

	virtual bool Visit(PostCondition*) { return _dive; }
	virtual void EndVisit(PostCondition*) { }

	virtual bool Visit(MemberAccess*) { return _dive; }
	virtual void EndVisit(MemberAccess*) { }

	virtual bool Visit(AndThen*) { return _dive; }
	virtual void EndVisit(AndThen*) { }

	virtual bool Visit(OrElse*) { return _dive; }
	virtual void EndVisit(OrElse*) { }

	virtual bool Visit(ReturnStatement*) { return _dive; }
	virtual void EndVisit(ReturnStatement*) { }

	virtual bool Visit(WhileStatement*) { return _dive; }
	virtual void EndVisit(WhileStatement*) { }

	virtual bool Visit(DoWhileStatement*) { return _dive; }
	virtual void EndVisit(DoWhileStatement*) { }

	virtual bool Visit(ForStatement*) { return _dive; }
	virtual void EndVisit(ForStatement*) { }

	virtual bool Visit(ForeachStatement*) { return _dive; }
	virtual void EndVisit(ForeachStatement*) { }

	virtual bool Visit(IfStatement*) { return _dive; }
	virtual void EndVisit(IfStatement*) { }

	virtual bool Visit(GlobalBlock*) { return _dive; }
	virtual void EndVisit(GlobalBlock*) { }

	virtual bool Visit(AssertStatement*) { return _dive; }
	virtual void EndVisit(AssertStatement*) { }

	virtual bool Visit(SeqLiteral*) { return _dive; }
	virtual void EndVisit(SeqLiteral*) { }

	virtual bool Visit(RecordLiteral*) { return _dive; }
	virtual void EndVisit(RecordLiteral*) { }

	virtual bool Visit(FieldAssign*) { return _dive; }
	virtual void EndVisit(FieldAssign*) { }

	virtual bool Visit(DataType*) { return _dive; }
	virtual void EndVisit(DataType*) { }

	virtual bool Visit(DataTypeDeclaration*) { return _dive; }
	virtual void EndVisit(DataTypeDeclaration*) { }

	virtual bool Visit(DataTypeCons*) { return _dive; }
	virtual void EndVisit(DataTypeCons*) { }

	virtual bool Visit(DataTypeLiteral*) { return _dive; }
	virtual void EndVisit(DataTypeLiteral*) { }

	virtual bool Visit(SwitchStatement*) { return _dive; }
	virtual void EndVisit(SwitchStatement*) { }

	virtual bool Visit(DataTypeCaseStatement*) { return _dive; }
	virtual void EndVisit(DataTypeCaseStatement*) { }

	virtual bool Visit(DefaultStatement*) { return _dive; }
	virtual void EndVisit(DefaultStatement*) { }

	virtual bool Visit(FunctionType*) { return _dive; }
	virtual void EndVisit(FunctionType*) { }

	virtual bool Visit(FunctionTypeDeclaration*) { return _dive; }
	virtual void EndVisit(FunctionTypeDeclaration*) { }

	virtual bool Visit(FunctionPointerCall*) { return _dive; }
	virtual void EndVisit(FunctionPointerCall*) { }

	virtual bool Visit(MakeFunctionPointer*) { return _dive; }
	virtual void EndVisit(MakeFunctionPointer*) { }

	virtual bool Visit(Enumerator*) { return _dive; }
	virtual void EndVisit(Enumerator*) { }

	virtual bool Visit(EnumType*) { return _dive; }
	virtual void EndVisit(EnumType*) { }

	virtual bool Visit(EnumTypeDeclaration*) { return _dive; }
	virtual void EndVisit(EnumTypeDeclaration*) { }

	virtual bool Visit(EnumCaseStatement*) { return _dive; }
	virtual void EndVisit(EnumCaseStatement*) { }

	virtual bool Visit(EnumLiteral*) { return _dive; }
	virtual void EndVisit(EnumLiteral*) { }

	virtual bool Visit(RecordUpdate*) { return _dive; }
	virtual void EndVisit(RecordUpdate*) { }
};

} // namespace wtast

#endif // WTPARSER_ASTVISITOR_H
