#include "config_private.h"
#include "objects.h"
#include "program.h"
#include "vm.h"
#include "debug.h"
#include <stddef.h>
#include <string.h>

wt_vmptr wtr_str_create_z(struct wtr_vm* const vm, const char* data)
{
	wt_u32 len;

	if(NULL!=data && 0!=(*data))
	{
		len = strlen(data);
	}
	else
	{
		len = 0;
	}

	return wtr_str_create_n(vm, data, len);
}

wt_vmptr wtr_str_create_n(struct wtr_vm* const vm, const char* data,
	wt_u32 len)
{
	wt_vmptr str;

	if(0!=(str=wtr_vm_vma_malloc(vm,
		sizeof(struct wtr_str)+len+OBJECT_MAGIC_LENGTH_FOR_STR)))
	{
		struct wtr_str m;

		m.length = len;

#ifdef ADD_INLINE_OBJECT_MAGIC
		m.magic1 = WTR_MAGIC_STR1;
		m.magic2 = WTR_MAGIC_STR2;
		m.length_save = m.length;
#endif /* ADD_INLINE_OBJECT_MAGIC */

		if(0!=wtr_vma_write(vm, str, &m, offsetof(struct wtr_str, data)) ||
			0!=wtr_vma_write(vm, str+offsetof(struct wtr_str, data)+len, "",
			1))
		{
		free_and_exit:;
			wtr_vm_vma_free(vm, str);
			return 0;
		}

		if(NULL!=data && 0<len)
		{
			if(0!=wtr_vma_write(vm, str+offsetof(struct wtr_str, data), data,
				len))
			{
				goto free_and_exit;
			}
		}

		SET_OBJECT_MAGIC_STR(str);
		LOG_ALLOC("STR", str, sizeof(struct wtr_str)+len);

		return str;
	}

	return 0;
}

void wtr_str_delete(struct wtr_vm* const vm, wt_vmptr str)
{
	LOG_FREE("STR", str);
	CHECK_OBJECT_MAGIC_STR(str);
#ifdef ADD_INLINE_OBJECT_MAGIC
	assert(wtr_str_is_valid(str));
#endif /* ADD_INLINE_OBJECT_MAGIC */
#ifdef WTRUNTIME_SAFE_OBJECT_DELETE
	wtr_vma_memset(vm, str, 0, offsetof(struct wtr_str, data));
#endif /* WTRUNTIME_SAFE_OBJECT_DELETE */
#ifdef ADD_INLINE_OBJECT_MAGIC
	wtr_vma_memset(vm, str+offsetof(struct wtr_str, length_save), -3,
		sizeof(((struct wtr_str*)0)->length_save));
#endif /* ADD_INLINE_OBJECT_MAGIC */
	wtr_vm_vma_free(vm, str);
}

wt_vmptr wtr_seq_create(struct wtr_vm* const vm, wt_u32 elemsize,
	wt_u32 count)
{
	const size_t seqsz = offsetof(struct wtr_seq, data);

	wt_vmptr seq = wtr_vm_vma_malloc(vm,
		seqsz+elemsize*count+OBJECT_MAGIC_LENGTH_FOR_SEQ);

	if(0!=seq)
	{
		struct wtr_seq m;

		m.length   = count;
		m.elemsize = elemsize;

#ifdef ADD_INLINE_OBJECT_MAGIC
		m.magic1 = WTR_MAGIC_SEQ1;
		m.magic2 = WTR_MAGIC_SEQ2;
		m.length_save = m.length;
#endif /* ADD_INLINE_OBJECT_MAGIC */

		if(0!=wtr_vma_write(vm, seq, &m, offsetof(struct wtr_seq, data)) ||
			0!=wtr_vma_memset(vm, seq+offsetof(struct wtr_seq, data), 0,
			elemsize*count))
		{
			wtr_vm_vma_free(vm, seq);
			return 0;
		}

		SET_OBJECT_MAGIC_SEQ(seq);
		LOG_ALLOC("SEQ", seq, seqsz+elemsize*count);

		return seq;
	}

	return 0;
}

void wtr_seq_delete(struct wtr_vm* const vm, wt_vmptr seq)
{
	LOG_FREE("SEQ", seq);
	CHECK_OBJECT_MAGIC_SEQ(seq);
#ifdef ADD_INLINE_OBJECT_MAGIC
	assert(wtr_seq_is_valid(seq));
#endif /* ADD_INLINE_OBJECT_MAGIC */
#ifdef WTRUNTIME_SAFE_OBJECT_DELETE
	wtr_vma_memset(vm, seq, 0, offsetof(struct wtr_seq, data));
#endif /* WTRUNTIME_SAFE_OBJECT_DELETE */
#ifdef ADD_INLINE_OBJECT_MAGIC
	wtr_vma_memset(vm, seq+offsetof(struct wtr_seq, length_save), -5,
		sizeof(((struct wtr_seq*)0)->length_save));
#endif /* ADD_INLINE_OBJECT_MAGIC */
	wtr_vm_vma_free(vm, seq);
}

wt_vmptr wtr_rec_create(struct wtr_vm* const vm, wt_u32 size)
{
	const size_t recsz = offsetof(struct wtr_rec, data);

	const wt_vmptr rec = wtr_vm_vma_malloc(vm,
			recsz+size+OBJECT_MAGIC_LENGTH_FOR_REC);

	if(0!=rec)
	{
		struct wtr_rec m;

		m.size = size;

#ifdef ADD_INLINE_OBJECT_MAGIC
		m.magic1 = WTR_MAGIC_REC1;
		m.magic2 = WTR_MAGIC_REC2;
		m.size_save = m.size;
#endif /* ADD_INLINE_OBJECT_MAGIC */

		if(0!=wtr_vma_write(vm, rec, &m, offsetof(struct wtr_rec, data)) ||
			0!=wtr_vma_memset(vm, rec+offsetof(struct wtr_rec, data), 0,
			size))
		{
			wtr_vm_vma_free(vm, rec);
			return 0;
		}

		SET_OBJECT_MAGIC_REC(seq);
		LOG_ALLOC("REC", rec, recsz+size);

		return rec;
	}

	return 0;
}

void wtr_rec_delete(struct wtr_vm* const vm, wt_vmptr rec)
{
	LOG_FREE("REC", rec);
	CHECK_OBJECT_MAGIC_REC(rec);
#ifdef ADD_INLINE_OBJECT_MAGIC
	assert(wtr_rec_is_valid(rec));
#endif /* ADD_INLINE_OBJECT_MAGIC */
#ifdef WTRUNTIME_SAFE_OBJECT_DELETE
	wtr_vma_memset(vm, rec, 0, offsetof(struct wtr_rec, data));
#endif /* WTRUNTIME_SAFE_OBJECT_DELETE */
#ifdef ADD_INLINE_OBJECT_MAGIC
	wtr_vma_memset(vm, rec+offsetof(struct wtr_rec, length_save), -7,
		sizeof(((struct wtr_rec*)0)->length_save));
#endif /* ADD_INLINE_OBJECT_MAGIC */
	wtr_vm_vma_free(vm, rec);
}

#ifdef ADD_INLINE_OBJECT_MAGIC

wt_bool wtr_str_is_valid(struct wtr_vm* const vm, wt_vmptr str)
{
	struct wtr_str m;

	if(0!=wtr_vma_read(vm, str, &m, offsetof(struct wtr_str, data)))
	{
		return WT_FALSE;
	}

	return WTR_MAGIC_STR1==m.magic1 && WTR_MAGIC_STR2==m.magic2 &&
		m.length==m.length_save;
}

wt_bool wtr_seq_is_valid(struct wtr_vm* const vm, wt_vmptr seq)
{
	struct wtr_seq m;

	if(0!=wtr_vma_read(vm, str, &m, offsetof(struct wtr_seq, data)))
	{
		return WT_FALSE;
	}

	return WTR_MAGIC_SEQ1==m.magic1 && WTR_MAGIC_SEQ2==n.magic2 &&
		m.length==m.length_save;
}

wt_bool wtr_rec_is_valid(struct wtr_vm* const vm, wt_vmptr rec)
{
	struct wtr_rec m;

	if(0!=wtr_vma_read(vm, str, &m, offsetof(struct wtr_rec, data)))
	{
		return WT_FALSE;
	}

	return WTR_MAGIC_REC1==m.magic1 && WTR_MAGIC_REC2==m.magic2 &&
		m.size==m.size_save;
}

#endif /* ADD_INLINE_OBJECT_MAGIC */
